import win32.windows;
import win32.windef;
import win32.commctrl;
import win32frame.WindowMenu;
import win32frame.StatusBar;
import win32frame.MainWindow ;
import win32frame.PushButton ;
import win32frame.TrackBar;
import win32frame.MsgBox ;
import win32frame.FileDialog ;
import std.stdio;
import std.string;
import std.utf;

import oggPlayer ;

enum MENUCODE{
  FILE_OGGOPEN = 0x101,
  FILE_EXIT    = 0x104,
  HELP_VERSION = 0x111
} ;

enum CMD{
  PLAYBTN  = 0x201,
  VOLSLIDE = 0x202
}

enum BTNSTAT{
  NON  =0x01,
  PLAY =0x02,
  STOP =0x03,
  PAUSE=0x04
} ;

const int ID_TIMER = 1 ;


class myMainWindow : MainWindow {
  WindowMenu   menuroot ;
  StatusBar    statusbar ;
  MsgBox       msgbox ;
  HFONT        hFont;
  
  oggPlayer    oggplayer ;
  PushButton   playbtn ;
  int          playbtnstat ;
  TrackBar     volume_slide ;

  this(string name, string ClassName) {
    winc.style          = 0;
    winc.cbClsExtra     = 0;
    winc.cbWndExtra     = 0;
    winc.hIcon          = LoadIcon(null , cast(wchar*)(IDI_APPLICATION));
    winc.hCursor        = LoadCursor(null , cast(wchar*)(IDC_ARROW));
    winc.hbrBackground  = CreateSolidBrush(GetSysColor(COLOR_BTNFACE)) ;
    winc.lpszMenuName   = null;
    winc.lpszClassName  = toUTFz!(wchar*)(ClassName) ;
    super(name,ClassName) ;
    
  }

  LRESULT proc(HWND hwnd , UINT msg , WPARAM wp , LPARAM lp){
    switch (msg) {
    case WM_DESTROY:
      DeleteObject(hFont);
      PostQuitMessage(0);
      break ;
      
    case WM_CREATE:
      menuroot = new WindowMenu() ;
      menuroot.create() ;
      menuroot.addMenu(         MFT_STRING, "File"      , 0x000 ) ;
      menuroot.addItem( "File", MFT_STRING, "Ogg Open"  , MENUCODE.FILE_OGGOPEN ) ;
      menuroot.addItem( "File", MFT_STRING, "Exit"      , MENUCODE.FILE_EXIT    ) ;
      menuroot.addMenu(         MFT_STRING, "Help"      , 0x001 ) ;
      menuroot.addItem( "Help", MFT_STRING, "Version"   , MENUCODE.HELP_VERSION ) ;
      SetMenu(hwnd , menuroot.getHMENU());
      
      statusbar = new StatusBar() ;
      statusbar.addStyle(CCS_BOTTOM | SBARS_SIZEGRIP) ;
      addStatusBar(statusbar, "") ;
      
      hFont = CreateFont(14 , 0 , 0 , 0 , FW_REGULAR , FALSE , FALSE , FALSE ,
			 SHIFTJIS_CHARSET , OUT_DEFAULT_PRECIS ,
			 CLIP_DEFAULT_PRECIS , DEFAULT_QUALITY , 
			 VARIABLE_PITCH, null
			 );
      
      DragAcceptFiles(hwnd,true) ;
      SetTimer(hwnd, ID_TIMER, 300, null) ;
      
      playbtn = new PushButton("non") ;
      playbtnstat=BTNSTAT.NON ;
      playbtn.setPosition(0,0,100,20) ;
      addObject(playbtn, cast(HMENU)(CMD.PLAYBTN), true, false) ;
      playbtn.setFont(hFont) ;

      volume_slide = new TrackBar("volume_slide") ;
      volume_slide.addStyle( TBS_NOTICKS ) ; //TBS_NOTHUMB | TBS_AUTOTICKS | TBS_ENABLESELRANGE | TBS_VERT ) ;
      volume_slide.setPosition(0,35,300,20) ;
      addObject(volume_slide, cast(HMENU)(CMD.VOLSLIDE), true,false) ;
      volume_slide.setRange(0,100) ;
      volume_slide.setPos(100) ;
      break ;

    case WM_TIMER:
      if( oggplayer !is null ){
	oggplayer.setGain( cast(double)(volume_slide.getPos())/100.0 ) ;
	oggplayer.polling() ;
	int curtime = cast(int)(oggplayer.getTime()) ;
	int curstat = oggplayer.getState() ;
	if( curstat==2 ){
	  oggplayer.pause() ;
	  oggplayer.rewind() ;
	  setPlayBtn(BTNSTAT.PLAY) ;
	}
	printStatusbar( format("%s : (%d sec)", (curstat==0 ? "一時停止" : curstat==1 ? "再生" : "停止"), curtime) ) ;
      }else{
	printStatusbar("") ;
      }
      break ;

    case WM_DROPFILES:
      int    filenum = DragQueryFile( cast(HDROP)(wp), -1, null, 0) ;
      wchar  wfname[] ;
      string fnamelist[] ;

      fnamelist.length=filenum ;
      for( int i=0 ; i<filenum ; i++ ){
	int namelen=DragQueryFile( cast(HDROP)(wp), i, null, 0) ;
	wfname.length=namelen+1 ;
	DragQueryFile( cast(HDROP)(wp), i, wfname.ptr, cast(int)(wfname.length)) ;
	wfname.length = wfname.length-1 ;
	fnamelist[i]=std.utf.toUTF8(wfname.idup) ;
      }
      DragFinish( cast(HDROP)(wp) ) ;
      openOggfile(fnamelist[0]) ;
      break ;

    case WM_COMMAND:
      switch( LOWORD(wp) ){
      case MENUCODE.FILE_OGGOPEN :
	FileDialog fdlog = new FileDialog() ;
	string fname   ;
	fname = fdlog.getLoadFileName("Ogg {*.ogg}\0*.ogg\0ALL {*.*}\0*.*\0",hwnd,OFN_HIDEREADONLY) ;
	openOggfile(fname) ;
	break ;
      case MENUCODE.FILE_EXIT :
	SendMessage( hwnd, WM_CLOSE , 0,0 ) ;
	break ;
      case MENUCODE.HELP_VERSION :
	msgbox = new MsgBox(hwnd, format("OggPlayer Ver0.01"), " ",MB_OK) ;
	msgbox.create() ;
	break ;
	
      case CMD.PLAYBTN :
	switch(playbtnstat){
	case BTNSTAT.PAUSE: //Pause→Play
	  oggplayer.pause() ;
	  setPlayBtn(BTNSTAT.PLAY) ;
	  break ;
	case BTNSTAT.PLAY: //Play→Pause
	  oggplayer.play() ;
	  setPlayBtn(BTNSTAT.PAUSE) ;
	  break ;
	default :
	  break ;
	}
	break ;
	
      default :	break ;
      }
      break;
      
    default :
      return( DefWindowProc(hwnd , msg , wp , lp) );
    }
    return(0);
  }

  int setPlayBtn(int stat){
    switch(stat){
    case BTNSTAT.NON:
      playbtn.setWindowText("non") ;
      playbtnstat=BTNSTAT.NON ;
      break ;
    case BTNSTAT.PLAY:
      playbtn.setWindowText("Play") ;
      playbtnstat=BTNSTAT.PLAY ;
      break ;
    case BTNSTAT.STOP:
      playbtn.setWindowText("Stop") ;
      playbtnstat=BTNSTAT.STOP ;
      break ;
    case BTNSTAT.PAUSE:
      playbtn.setWindowText("Pause") ;
      playbtnstat=BTNSTAT.PAUSE ;
      break ;
    default :
      core.exception.onSwitchError() ;
    }
    return(0) ;
  }

  
  int openOggfile(string fname){
    if( fname.length != 0 ){
      if( oggplayer !is null ){
	oggplayer.pause() ;
	delete oggplayer ; oggplayer=null ;
	
      }
      oggplayer = new oggPlayer() ;
      oggplayer.init() ;
      
      try{
	oggplayer.open(fname) ;
	oggplayer.play() ;
	setPlayBtn(BTNSTAT.PAUSE) ;
      }catch(Exception e){
	delete oggplayer ; oggplayer=null ;
	msgbox = new MsgBox(getHWND(), format("ファイルのオープンに失敗しました(%s)",fname), " ",MB_OK) ;
	msgbox.create() ;
	setPlayBtn(BTNSTAT.NON) ;
      }
    }
    return(0) ;
  }
  
}
