import win32.windows;
import std.stdio;
import std.string;
import std.stream;
import std.utf;
import alut;

struct OggVorbis_File{
  int padding[210] ;
} ;
  
struct vorbis_info{
  int   _version ;
  int   channels ;
  int   rate     ;
  int   bitrate_upper   ;
  int   bitrate_nominal ;
  int   bitrate_lower   ;
  int   bitrate_window  ;
  void  *coldec_setup   ;
} ;

extern (C) int    ov_fopen(const char *path, OggVorbis_File *vf) ;
extern (C) int    ov_wfopen(const wchar *path, OggVorbis_File *vf) ;
extern (C) vorbis_info *ov_info(OggVorbis_File *vf,int link);
extern (C) int    ov_read(OggVorbis_File *vf,void *buffer,int length, int bigendianp,int word,int sgned,int *bitstream);
extern (C) int    ov_clear(OggVorbis_File *vf);
extern (C) double ov_time_tell(OggVorbis_File *vf);
extern (C) double ov_time_total(OggVorbis_File *vf, int i);
extern (C) int    ov_time_seek(OggVorbis_File *vf,double pos);

enum STATE{
  PAUSE = 0,
  PLAY  = 1,
  STOP  = 2
} ;


class oggPlayer
{
  OggVorbis_File  vf;
  vorbis_info    *vi;
  int      current  ;
  
  ALuint   source   ;
  ALuint[] buffer  ;
  int      usedbuf  ;
  ALenum   format   ;
  ubyte[]  pcmdata  ;
  
  int      initialize ;
  bool     opened   ;
  STATE    state    ;
  bool     loopplay ;
  double   gain     ;

  static int refcount ;
  
  this(){
    opened     = false ;
    initialize = 0 ;
    loopplay   = false ;
    state      = STATE.PAUSE ;
    gain       = 1.0 ;
    refcount++ ;
  }

  ~this(){
    this.close() ;
    refcount-- ;
    if( refcount==0 ) alutExit() ;
  }
  
  int init(){
    alutInit(null, null);
    synchronized{initialize=1 ;}
    return(0) ;
  }
  
  int open(string fname){
    if( opened==true ){
      throw new Exception( std.string.format("Class oggPlayer.open(%s) multi called",fname) ) ;
    }
    int ret = ov_wfopen( toUTFz!(wchar*)(fname), &vf) ;
    if( ret<0 ){
      throw new Exception( std.string.format("Class oggPlayer.open(%s) failed",fname) ) ;
    }
    vi=ov_info(&vf,-1);
    format = vi.channels == 1 ? AL_FORMAT_MONO16 : AL_FORMAT_STEREO16;
    
    alGenSources(1,&source) ;
    
    pcmdata.length=2048 ;
    buffer.length=64 ;
    usedbuf=0 ;
    
    opened=true ;
    return(0) ;
  }
  
  int polling(){
    if( opened==false ) return(0) ;
    
    ALint  st, n;
    int    size ;
    
    alGetSourcei(source, AL_BUFFERS_PROCESSED, &n) ;
    if( n!=0 ){
      for( int i=0 ; i<n ; i++ ){
	alSourceUnqueueBuffers(source, 1, &buffer[i]);
	alDeleteBuffers(1, &buffer[i]) ;
      }
      for( int i=0 ; i<buffer.length-n ; i++ ){
	buffer[i]=buffer[n+i] ;
      }
      usedbuf-=n ;
    }
    
    alGetSourcei(source, AL_BUFFERS_QUEUED, &n);
    for( int i=0 ; i<(buffer.length-n) ; i++ ){
      size = ov_read(&vf, pcmdata.ptr, cast(int)(pcmdata.length), 0,2,1,&current) ;
      if( size<=0 ){
	if( loopplay==true ){
	  rewind() ;
	}else{
	  state=STATE.STOP ;
	}
	break ;
      }
      alGenBuffers(1, &buffer[usedbuf]);
      //      writef("size = %d usedbuf=%d time=%f\n",size,usedbuf, ov_time_tell(&vf)) ;
      alBufferData(buffer[usedbuf], format, pcmdata.ptr, size, vi.rate);
      alSourceQueueBuffers(source, 1, &buffer[usedbuf]);
      usedbuf = usedbuf+1 ;
    }
    alSourcef( source, AL_GAIN, gain ) ;
    
    switch( state ){
    case STATE.PLAY :
      alGetSourcei(source, AL_SOURCE_STATE, &st);
      if( st == AL_STOPPED && size<=0 && loopplay==false ){
	state=STATE.STOP ;
	break ;
      }
      if( st != AL_PLAYING ){
	alSourcePlay(source);
      }
      break ;
    case STATE.PAUSE :
      alSourcePause(source) ;
      break ;
    default :
      break ;
    }
    
    return(0) ;
  }
  
  int play(){
    state = STATE.PLAY ;
    return(0) ;
  }
  
  int pause(){
    state = STATE.PAUSE ;
    return(0) ;
  }
  
  int rewind(){
    if( opened==false ){
      return(0) ;
      //      throw new Exception( std.string.format("Class oggPlayer.polling() not yet called open()") ) ;
    }
    if( vi!=null ){
      ov_time_seek(&vf,0.0) ;
    }
    return(0) ;
  }

  int setGain(double g){
    gain = g ;
    return(0) ;
  }

  
  int getState(){
    return(state) ;
  }
  
  int setLoopPlay(bool cmd){
    loopplay=cmd ;
    return(0) ;
  }
  
  double getTime(){
    if( opened==false ){
      return(double.nan) ;
      //      throw new Exception( std.string.format("Class oggPlayer.polling() not yet called open()") ) ;
    }
    return(ov_time_tell(&vf)) ;
  }
  
  int close(){
    if( opened==true ){
      ov_clear(&vf) ;
      alSourceStop(source) ;
      for( int i=0 ; i<usedbuf ; i++ ){
	alSourceUnqueueBuffers(source, 1, &buffer[i]);
	alDeleteBuffers(1, &buffer[i]) ;
      }
      alDeleteSources(1, &source);

      delete pcmdata ;
      delete buffer  ;
    }
    return(0) ;
  }
  
}
  
